//  To use Hierarchical Data Control Library controls in your projects follow these steps:
//
//  1. Create or open a Windows Forms Application project in Visual Studio 2005/2008.
//
//  2. Add reference to Hierarchical Data Control Library assembly. To do this, right 
//     click the Windows Forms Application project in Visual Studio's Solution Explorer, and 
//     select Add Reference from the context menu. Select the Browse tab from the Add 
//     Reference dialog to find and select Hierarchical Data Control Library assembly, 
//     usually installed as C:\Program Files\DlhSoft\Hierarchical Data Control Library 2.1\
//     DlhSoft.HierarchicalDataControlLibrary.dll file. The reference is added to your project.
//
//  3. If you want to use the controls at design time, add the Hierarchical Data 
//     Control Library control icons to the Visual Studio form designer toolbar. To do 
//     this, open a Windows Form within Visual Studio in design mode, right click a 
//     Toolbox tab and select Choose Items command from the context menu. In the 
//     Choose Toolbox Items dialog use the Browse button to find and select 
//     Hierarchical Data Control Library assembly, usually installed as 
//     C:\Program Files\DlhSoft\Hierarchical Data Library 2.0.0\
//     DlhSoft.HierarchicalDataControlLibrary.dll file. Make sure that all the 
//     controls you want are selected, and click OK. Find the control icons within the 
//     toolbox. The initially selected control is DataTreeGridView.
//
//  4. Use the Hierarchical Data Control Library controls according to the technical 
//     documentation available from Start Menu, DlhSoft Hierarchical Data Control 
//     Library 2.1, Documentation.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace HierarchicalDataControlLibrarySample
{
    public partial class MyForm : Form
    {
        #region Constructors

        public MyForm()
        {
            InitializeComponent();

            //By default, open the objects binding mode and initialize data
            tabControlModes.SelectedTab = objectsTabPage;
            objectsReinitializeButton_Click(this, EventArgs.Empty);
        }

        #endregion

        #region Data Initialization

        //No binding
        private void standardReinitializeButton_Click(object sender, EventArgs e)
        {
            CurrentDataBindingMode = DataBindingMode.Standard;

            dataTreeGridView.Rows.Clear();
            
            dataTreeGridView.Rows.Add(1, "Node 1", "Description for node 1");
            dataTreeGridView.Rows.Add(2, "Node 2", "Description for node 2");
            dataTreeGridView.Rows.Add(3, "Node 3", "Description for node 3");
            dataTreeGridView.Rows.Add(4, "Node 4", "Description for node 4");
            dataTreeGridView.Rows.Add(5, "Node 5", "Description for node 5");

            dataTreeGridView.IncreaseIndent(1); //Node 2
            dataTreeGridView.IncreaseIndent(2); //Node 3
            dataTreeGridView.IncreaseIndent(4); //Node 5

            dataTreeGridView.SetImageIndex(0, 1); //Node 1
            dataTreeGridView.SetImageIndex(3, 1); //Node 4
            
            dataTreeGridView.Close(0); //Node 1

            for (int i = 6; i <= 20; i++)
                dataTreeGridView.Rows.Add(i, "Node " + i, "Description for node " + i);

            ensureEnableUI();
        }

        //DataTable binding
        private void dataTableReinitializeButton_Click(object sender, EventArgs e)
        {
            CurrentDataBindingMode = DataBindingMode.DataTable;

            DataTable dataTable = new DataTable("MyDataTable");
            dataTable.Columns.Add("ID", typeof(int));
            dataTable.Columns.Add("Name", typeof(string));
            dataTable.Columns.Add("Description", typeof(string));
            dataTable.Columns.Add("Expanded", typeof(bool));
            dataTable.Columns.Add("Indent", typeof(int));
            dataTable.Columns.Add("Image", typeof(int));

            dataTable.Columns["Expanded"].DefaultValue = true;
            dataTable.Columns["Indent"].DefaultValue = 0;
            dataTable.Columns["IMage"].DefaultValue = 0;

            dataTable.Rows.Add(1, "Node 1", "Description for node 1", false, 0, 1);
            dataTable.Rows.Add(2, "Node 2", "Description for node 2", true, 1, 0);
            dataTable.Rows.Add(3, "Node 3", "Description for node 3", true, 1, 0);
            dataTable.Rows.Add(4, "Node 4", "Description for node 4", true, 0, 1);
            dataTable.Rows.Add(5, "Node 5", "Description for node 5", true, 1, 0);

            for (int i = 6; i <= 20; i++)
                dataTable.Rows.Add(i, "Node " + i, "Description for node " + i, true, 0, 0);

            dataTreeGridView.DataSource = dataTable;

            ensureEnableUI();
        }

        //Object binding
        private void objectsReinitializeButton_Click(object sender, EventArgs e)
        {
            CurrentDataBindingMode = DataBindingMode.Objects;

            BindingList<MyNode> nodes = new BindingList<MyNode>();

            nodes.Add(new MyNode(1, "Node 1", "Description for node 1", false, 0, 1));
            nodes.Add(new MyNode(2, "Node 2", "Description for node 2", true, 1, 0));
            nodes.Add(new MyNode(3, "Node 3", "Description for node 3", true, 1, 0));
            nodes.Add(new MyNode(4, "Node 4", "Description for node 4", true, 0, 1));
            nodes.Add(new MyNode(5, "Node 5", "Description for node 5", true, 1, 0));

            for (int i = 6; i <= 20; i++)
                nodes.Add(new MyNode(i, "Node " + i, "Description for node " + i, true, 0, 0));

            dataTreeGridView.DataSource = nodes;

            ensureEnableUI();
        }

        #endregion

        #region Binding Mode

        //The sample can run in three modes, selected from the tab control: 
        //Standard (no binding), DataTable (DataTable binding), and Objects (object binding)
        //When the user hasn't initialized data in any of the available modes, the NotSelected mode is used.
        private enum DataBindingMode
        {
            NotSelected,
            Standard,
            DataTable,
            Objects
        }

        private DataBindingMode currentDataBindingMode;

        //When the current databinding mode changes, we make sure to enable the appropriate user interface controls
        private DataBindingMode CurrentDataBindingMode
        {
            get { return currentDataBindingMode; }
            set 
            { 
                currentDataBindingMode = value;

                //Only the Initialize button of the current databinding mode is available;
                //When the user hasn't yet initialized data in a specific mode, all Initialize buttons are available.
                standardReinitializeButton.Enabled = CurrentDataBindingMode == DataBindingMode.Standard || CurrentDataBindingMode == DataBindingMode.NotSelected;
                dataTableReinitializeButton.Enabled = CurrentDataBindingMode == DataBindingMode.DataTable || CurrentDataBindingMode == DataBindingMode.NotSelected;
                objectsReinitializeButton.Enabled = CurrentDataBindingMode == DataBindingMode.Objects || CurrentDataBindingMode == DataBindingMode.NotSelected;

                //The Clear button is available only for the current databinding mode.
                standardClearButton.Enabled = CurrentDataBindingMode == DataBindingMode.Standard;
                dataTableClearButton.Enabled = CurrentDataBindingMode == DataBindingMode.DataTable;
                objectsClearButton.Enabled = CurrentDataBindingMode == DataBindingMode.Objects;

                //The grid is enabled only after initializing data.
                dataTreeGridView.Enabled = CurrentDataBindingMode > DataBindingMode.NotSelected;

                //Special databinding grid columns are visible only in databound modes (DataTable or Objects).
                indentDataGridViewColumn.Visible = CurrentDataBindingMode > DataBindingMode.Standard;
                iconDataGridViewColumn.Visible = CurrentDataBindingMode > DataBindingMode.Standard;
                expandDataGridViewColumn.Visible = CurrentDataBindingMode > DataBindingMode.Standard;

                //Sorting allowed only in Objects databound mode.
                idDataGridViewColumn.SortMode = CurrentDataBindingMode == DataBindingMode.Objects ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
                nameDataGridViewColumn.SortMode = CurrentDataBindingMode == DataBindingMode.Objects ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
                descriptionDataGridViewColumn.SortMode = CurrentDataBindingMode == DataBindingMode.Objects ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
                iconDataGridViewColumn.SortMode = CurrentDataBindingMode == DataBindingMode.Objects ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;

                //Disabled modes notification
                standardDisabledLabel.Visible = CurrentDataBindingMode != DataBindingMode.Standard && CurrentDataBindingMode != DataBindingMode.NotSelected;
                dataTableDisabledLabel.Visible = CurrentDataBindingMode != DataBindingMode.DataTable && CurrentDataBindingMode != DataBindingMode.NotSelected;
                objectsDisabledLabel.Visible = CurrentDataBindingMode != DataBindingMode.Objects && CurrentDataBindingMode != DataBindingMode.NotSelected;
            }
        }

        #endregion

        #region Item Navigation

        //When the current cell changes (the user navigates in the grid), we need to make sure the appropriate user interface controls are enabled
        private void dataTreeGridView_CurrentCellChanged(object sender, EventArgs e)
        {
            //We delegate the functionality on a timer, to make sure it doesn't occur multiple times
            //when the CurrentCellChanged occurs multiple times in a short time
            timerCurrentCellChanged.Stop(); //reset timer interval if already started
            timerCurrentCellChanged.Start();
        }

        private void timerCurrentCellChanged_Tick(object sender, EventArgs e)
        {
            timerCurrentCellChanged.Stop(); //stop timer
            ensureEnableUI();
        }

        //After cell is edited changes, we need to make sure the appropriate user interface controls are enabled,
        //because the user might have just changed a property such as Is Expanded or IndentLevel.
        private void dataTreeGridView_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            ensureEnableUI(e.RowIndex);
        }

        //Make sure to enable the appropriate user interface controls based on the currently selected row index
        private void ensureEnableUI()
        {
            int rowIndex = dataTreeGridView.CurrentRow != null ? dataTreeGridView.CurrentRow.Index : -1;
            ensureEnableUI(rowIndex);
        }

        //Make sure to enable the appropriate user interface controls based on the specified selected row index
        private void ensureEnableUI(int rowIndex)
        {
            switch (CurrentDataBindingMode)
            {
                case DataBindingMode.Standard:
                    standardExpandButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && (dataTreeGridView.CanExpand(rowIndex) || dataTreeGridView.CanClose(rowIndex));
                    standardIncreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanIncreaseIndent(rowIndex);
                    standardDecreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanDecreaseIndent(rowIndex);
                    standardSetIconLabel.Enabled = standardSetIconComboBox.Enabled = rowIndex >= 0 && rowIndex != dataTreeGridView.NewRowIndex && dataTreeGridView.Rows[rowIndex].Visible;
                    standardSetIconComboBox.SelectedIndex = -1;
                    break;

                case DataBindingMode.DataTable:
                    dataTableExpandButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && (dataTreeGridView.CanExpand(rowIndex) || dataTreeGridView.CanClose(rowIndex));
                    dataTableIncreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanIncreaseIndent(rowIndex);
                    dataTableDecreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanDecreaseIndent(rowIndex);
                    dataTableSetIconLabel.Enabled = dataTableSetIconComboBox.Enabled = rowIndex >= 0 && rowIndex != dataTreeGridView.NewRowIndex && dataTreeGridView.Rows[rowIndex].Visible;
                    dataTableSetIconComboBox.SelectedIndex = -1;
                    break;

                case DataBindingMode.Objects:
                    objectsExpandButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && (dataTreeGridView.CanExpand(rowIndex) || dataTreeGridView.CanClose(rowIndex));
                    objectsIncreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanIncreaseIndent(rowIndex);
                    objectsDecreaseIndentButton.Enabled = rowIndex >= 0 && dataTreeGridView.Rows[rowIndex].Visible && dataTreeGridView.CanDecreaseIndent(rowIndex);
                    objectsSetIconLabel.Enabled = objectsSetIconComboBox.Enabled = rowIndex >= 0 && rowIndex != dataTreeGridView.NewRowIndex && dataTreeGridView.Rows[rowIndex].Visible;
                    objectsSetIconComboBox.SelectedIndex = -1;
                    break;
            }
        }

        #endregion

        #region Current Item Actions

        private void expandButton_Click(object sender, EventArgs e)
        {
            int rowIndex = dataTreeGridView.CurrentRow.Index;
            if (dataTreeGridView.IsClosed(rowIndex))
                dataTreeGridView.Expand(rowIndex);
            else
                dataTreeGridView.Close(rowIndex);
            ensureEnableUI(rowIndex);
        }

        private void increaseIndentButton_Click(object sender, EventArgs e)
        {
            int rowIndex = dataTreeGridView.CurrentRow.Index;
            dataTreeGridView.IncreaseIndent(rowIndex);
            ensureEnableUI(rowIndex);
        }

        private void decreaseIndentButton_Click(object sender, EventArgs e)
        {
            int rowIndex = dataTreeGridView.CurrentRow.Index;
            dataTreeGridView.DecreaseIndent(rowIndex);
            ensureEnableUI(rowIndex);
        }

        private void setIconComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBox = sender as ComboBox;
            if (comboBox.SelectedIndex >= 0)
            {
                int rowIndex = dataTreeGridView.CurrentRow.Index;
                dataTreeGridView.SetImageIndex(rowIndex, comboBox.SelectedIndex);
            }
        }

        #endregion

        #region Data Clear

        private void clearButton_Click(object sender, EventArgs e)
        {
            dataTreeGridView.DataMember = string.Empty;
            dataTreeGridView.DataSource = null;
            dataTreeGridView.Rows.Clear();

            CurrentDataBindingMode = DataBindingMode.NotSelected;
        }

        #endregion

        #region Properties and Close

        private void propertiesButton_Click(object sender, EventArgs e)
        {
            propertiesButton.Enabled = false;

            PropertiesForm propertiesForm = new PropertiesForm();
            propertiesForm.Location = new Point(this.Left + this.Width, this.Top);
            if (propertiesForm.Left + propertiesForm.Width > Screen.GetWorkingArea(propertiesForm).Width)
                propertiesForm.Left = Screen.GetWorkingArea(propertiesForm).Width - propertiesForm.Width;
            propertiesForm.Height = this.Height;
            propertiesForm.DataTreeGridView = this.dataTreeGridView;
            propertiesForm.FormClosed += new FormClosedEventHandler(propertiesForm_FormClosed);
            propertiesForm.Show(this);
        }

        private void propertiesForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            propertiesButton.Enabled = true;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}